function PPXA_Gaussian_TV(x,niter,z,T,L,alpha,nomImage,gamma,lambda,coef,minx,maxx) %#ok<FNDEF>


% INITIALIATION
%----------------

Nx=length(x);           % Image size  Nx^2    
mu=1;

%Number of functional 
%--------------------
flou = 1;       %Data fidelity term
cc = 1;         %Convex constraint
tv=4;           %Total variation [Combettes&Pesquet2008]

N1=flou +cc +tv;
fprintf('\n Proximity operators number \t: %d\n',N1);
fprintf('----------------------------\n');
fprintf('Proximity operators related to data fidelity term \t: %d\n',flou);
fprintf('Projection onto the convex constraint \t: %d\n',cc);
fprintf('Proximity operators related to TV \t: %d\n',tv);


%Algorithm initialization
%------------------------
xrec=z;
zl = z(:)';

yni = ones(N1,1)*zl;
pni = ones(N1,1)*zeros(size(zl));


%Weighted constants initialization
%---------------------------------
wi = ones(N1,1)/(L*L*3);
wi(1)=1/3;
wi(end-3:end)=1/(4*3);
wi(end-4)=1/3;


if (sum(wi)>1.0001 || sum(wi)<0.9999 ) 
    fprintf('ERREUR: The wi''s is non equal to 1. The value is %3.7f\n\n',sum(wi));
end

% wxn initialization
%-------------------
xn_temp = (wi*ones(1,length(yni))).*yni;
xn = sum(xn_temp,1);



% Criterion initialization
%-------------------------
snr = zeros(1,niter);
distQUAD=zeros(1,niter);
tv=zeros(1,niter);
dc=zeros(1,niter);
critere=zeros(1,niter);
time = zeros(1,niter+1);

fprintf('\n Start PPXA \n'); 
fprintf('------------- \n');

%Algorithm
%---------
time(1)=0;
for n=1:niter
    tic;
   
   
    distQUAD(n) = quad(z,xrec,T,alpha) ;
    snr(n) = snrdeblur(x,xrec);
    tv(n) = Variation_totale(xrec) ;
    dc(n) = distance_convexe(xrec,minx,maxx);
    critere(n) = distQUAD(n) + coef*tv(n);
    if(mod(n,10)==0) 
        fprintf('%d : dc = %3.2f  |  critere = %3.2f |  snr = %3.2f \n',n,dc(n),critere(n), snr(n));
    end
    
    
    % Proximity operators computation
    %--------------------------------
    
    tic;  
    
    %relative to data fidelity term
    ind=1; 
    [pni(ind,:)]=prox_quad(yni(ind,:),Nx,z,T,alpha,wi(ind,1),gamma);
      
    
    
    % relative to convex constraint projection
    ind = flou + 1;
    [pni(ind,:)]=projectionDTT(yni(ind,:),minx,maxx);


       
    % relative to total variation  
    [pni(N1-3,:)] = prox_tv_partiel(yni(N1-3,:),Nx,0,0,wi(N1-3,:),gamma,coef,mu);
    [pni(N1-2,:)] = prox_tv_partiel(yni(N1-2,:),Nx,0,1,wi(N1-2,:),gamma,coef,mu);
    [pni(N1-1,:)] = prox_tv_partiel(yni(N1-1,:),Nx,1,0,wi(N1-1,:),gamma,coef,mu);
    [pni(N1,:)] = prox_tv_partiel(yni(N1,:),Nx,1,1,wi(N1,:),gamma,coef,mu);


    
    % weighted sum
    %-------------   
    pn_temp = (wi*ones(1,length(pni))).*pni;   
    pn = sum(pn_temp,1);

    
    
    % yn update
    %----------
    yni = yni + lambda*(2*(ones(N1,1)*pn) - ones(N1,1)*xn - pni);
    
 
    % xn update
    %---------- 
    xn = xn + lambda * (pn - xn);
    
 
    time(n+1) = time(n)+ toc;  
    
    xrec = reshape(xn,Nx,Nx);

end
fprintf('\n\n PPXA tv:\n');
fprintf('----------\n');
fprintf('time = %5.1f  \n',time(n+1));
fprintf('snr = %3.2f\n',snr(n));
fprintf('ssim = %3.3f\n',ssim(x,xrec));
fprintf('distance to C = %3.2f  \n',dc(n));

figure(1)
subplot(131)
imagesc(x);
axis image off
subplot(132)
imagesc(z);
axis image off
subplot(133)
imagesc(xrec);
axis image off
colormap(gray)
figure(2)
subplot(311)
plot(snr)
title('SNR')
subplot(312)
plot(critere)
title('Objective function')
subplot(313)
plot(dc)
title('Distance to C')


%%%%%%%%%%%%%%%%% SAVE DATA %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
tit=['PPXA-GaussianDeblur-TV-',nomImage,'-niter',int2str(niter),'alpha',num2str(alpha),'L',int2str(L),'coef_tv',num2str(coef),'gamma',num2str(gamma),'lambda',num2str(lambda),'.mat'];
save(tit,'x','z','xrec','snr','critere','time','distQUAD','tv','N1','dc');
